﻿/****************************************************************************
**
** Copyright (C) 2010-2011 Iteco Trading Srl
** All rights reserved.
** Contact: Iteco Trading Srl (info@itecad.it)
**
** This file is part of the Itecad EDA software.
**
** Itecad is free software. You can redistribute it and/or modify it under the terms of
** the GNU Lesser General Public License (LGPLv2.1) as published by the Free Software Foundation
**
** This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
** See the GNU Lesser General Public License for more details.
**
** You should have received a copy of the GNU Lesser General Public License along with this software;
** if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
** If you have questions regarding the use of this file, please contact
** Iteco Trading Srl at info@itecad.it
**
****************************************************************************/

#include "win_library_info.h"

extern	win_main	*w_main;

extern	QString directory_ssf;
extern	QString directory_pff;
extern	QString directory_ivf;

win_library_info::win_library_info(QWidget *parent, Qt::WindowFlags tipo)
    : QDialog(parent, tipo)
{
    // Se questo QDialog viene creato con tipo == Qt::Dialog allora sono costretto ad avere il ? in alto a destra
    // Ciò forse è un bene perchè mi costringerà ad avere un minimo di help
    // Veder Qt::WindowContextHelpButtonHint per avere maggiori informazioni

    setWindowTitle(tr("Library Info"));
    //setSizeGripEnabled(true);
    setModal(true);

    //Imposto la larghezza minima pari a metà dello schermo
    QRect screen = QApplication::desktop()->screenGeometry();	//Non serve perchè uso tw->setUsesScrollButtons(FALSE);
    setMinimumWidth(screen.width() / 2);						//Non serve perchè uso tw->setUsesScrollButtons(FALSE);

    QGridLayout	*layout = new QGridLayout(this);

    layout->addWidget(setup_directories(),1,0);
    layout->addLayout(setup_buttons(),2,0);
    layout->setSpacing(6);
    layout->setMargin(8);

}


QGroupBox *win_library_info::setup_directories()
{
    QGroupBox *directories = new QGroupBox(tr("Directories"));
    QLabel *dir_ssf = new QLabel(this);
    dir_ssf->setText(tr("Schematic symbols directory:"));
    QHBoxLayout *riga2 = new QHBoxLayout(0);
    QLineEdit *dir_SSF = new QLineEdit(QDir::toNativeSeparators(directory_ssf));
    dir_SSF->setDisabled(TRUE);
    QToolButton	*dir_ssf_change = new QToolButton(this);
    //dir_ssf_change->setText(tr("Browse"));
    dir_ssf_change->setIcon(QIcon(":/Resources/folder_explore.png"));
    dir_ssf_change->setAutoRaise(TRUE);
    dir_ssf_change->setCheckable(FALSE);
    connect(dir_ssf_change,SIGNAL(clicked()),this,SLOT(change_ssf_directory()));
    riga2->addSpacing(30);
    riga2->addWidget(dir_SSF);
    riga2->addWidget(dir_ssf_change);

    QLabel *dir_pff = new QLabel(this);
    dir_pff->setText(tr("PCB footprints directory:"));
    QHBoxLayout *riga3 = new QHBoxLayout(0);
    QLineEdit *dir_PFF = new QLineEdit(QDir::toNativeSeparators(directory_pff));
    dir_PFF->setDisabled(TRUE);
    QToolButton	*dir_pff_change = new QToolButton(this);
    //dir_pff_change->setText(tr("Browse"));
    dir_pff_change->setIcon(QIcon(":/Resources/folder_explore.png"));
    dir_pff_change->setAutoRaise(TRUE);
    dir_pff_change->setCheckable(FALSE);
    connect(dir_pff_change,SIGNAL(clicked()),this,SLOT(change_pff_directory()));
    riga3->addSpacing(30);
    riga3->addWidget(dir_PFF);
    riga3->addWidget(dir_pff_change);

    QLabel *dir_ivf = new QLabel(this);
    dir_ivf->setText(tr("Internal Vector Font directory:"));
    QHBoxLayout *riga4 = new QHBoxLayout(0);
    QLineEdit *dir_IVF = new QLineEdit(QDir::toNativeSeparators(directory_ivf));
    dir_IVF->setDisabled(TRUE);
    QToolButton	*dir_ivf_change = new QToolButton(this);
    //dir_pff_change->setText(tr("Browse"));
    dir_ivf_change->setIcon(QIcon(":/Resources/folder_explore.png"));
    dir_ivf_change->setAutoRaise(TRUE);
    dir_ivf_change->setCheckable(FALSE);
    connect(dir_ivf_change,SIGNAL(clicked()),this,SLOT(change_ivf_directory()));
    riga4->addSpacing(30);
    riga4->addWidget(dir_IVF);
    riga4->addWidget(dir_ivf_change);

    QGridLayout *gbox = new QGridLayout;
    gbox->addWidget(dir_ssf,1,0);
    gbox->addLayout(riga2,2,0,TRUE,FALSE);
    QSpacerItem *spazio1 = new QSpacerItem(20,20);
    gbox->addItem(spazio1,3,0);
    gbox->addWidget(dir_pff,4,0);
    gbox->addLayout(riga3,5,0,TRUE,FALSE);
    QSpacerItem *spazio2 = new QSpacerItem(20,20);
    gbox->addItem(spazio2,6,0);
    gbox->addWidget(dir_ivf,7,0);
    gbox->addLayout(riga4,8,0,TRUE,FALSE);
    gbox->setRowStretch(6,TRUE);

    directories->setLayout(gbox);

    return directories;
}

QHBoxLayout	*win_library_info::setup_buttons()
{
    QHBoxLayout	*L_comandi = new QHBoxLayout();
    L_comandi->setSpacing(6);
    L_comandi->setMargin(0);
    QSpacerItem	*L_comandi_molla = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);
    L_comandi->addItem(L_comandi_molla);
    QPushButton	*L_comandi_OK = new QPushButton(this);
    L_comandi_OK->setText(tr("Ok"));
    L_comandi->addWidget(L_comandi_OK);
    QPushButton	*L_comandi_Cancel = new QPushButton(this);
    L_comandi_Cancel->setText(tr("Cancel"));
    L_comandi->addWidget(L_comandi_Cancel);

    QObject::connect(L_comandi_OK, SIGNAL(clicked()), this, SLOT(accept()));
    QObject::connect(L_comandi_Cancel, SIGNAL(clicked()), this, SLOT(reject()));

    L_comandi_OK->setAutoDefault(TRUE);
    L_comandi_Cancel->setAutoDefault(FALSE);

    return L_comandi;
}

void	win_library_info::change_ssf_directory()
{
    QString path = QDir::toNativeSeparators(QApplication::applicationDirPath());
    QDesktopServices::openUrl(QUrl("file:///" + directory_ssf));
}

void	win_library_info::change_pff_directory()
{
    QString path = QDir::toNativeSeparators(QApplication::applicationDirPath());
    QDesktopServices::openUrl(QUrl("file:///" + directory_pff));
}

void	win_library_info::change_ivf_directory()
{
    QString path = QDir::toNativeSeparators(QApplication::applicationDirPath());
    QDesktopServices::openUrl(QUrl("file:///" + directory_ivf));
}

void	win_library_info::accept()
{
    QDialog::accept();
}
