﻿/****************************************************************************
**
** Copyright (C) 2010-2011 Iteco Trading Srl
** All rights reserved.
** Contact: Iteco Trading Srl (info@itecad.it)
**
** This file is part of the Itecad EDA software.
**
** Itecad is free software. You can redistribute it and/or modify it under the terms of
** the GNU Lesser General Public License (LGPLv2.1) as published by the Free Software Foundation
**
** This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
** See the GNU Lesser General Public License for more details.
**
** You should have received a copy of the GNU Lesser General Public License along with this software;
** if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
**
** If you have questions regarding the use of this file, please contact
** Iteco Trading Srl at info@itecad.it
**
****************************************************************************/

#include "win_move_to.h"

extern	myGraphicsView	*GV;
extern	win_main	*w_main;
extern  bool    y_axis_is_up;

extern	QPoint	drag_or_move_delta;
extern	bool	mode_drag_active;

extern  QList<myGraphicsItem *>     clipboard_obj;
extern  QList<myComponent *>        clipboard_comp;
extern  QList<node_data *>          clipboard_nodes;

win_move_to::win_move_to(QWidget *parent, Qt::WindowFlags tipo)
    : QDialog(parent, tipo)
{
    // Se questo QDialog viene creato con tipo == Qt::Dialog allora sono costretto ad avere il ? in alto a destra
    // Ciò forse è un bene perchè mi costringerà ad avere un minimo di help			//Non serve perchè uso tw->setUsesScrollButtons(FALSE);
    // Veder Qt::WindowContextHelpButtonHint per avere maggiori informazioni		//Non serve perchè uso tw->setUsesScrollButtons(FALSE);

    setWindowTitle(w_main->edit_positionAct->text());
    //setSizeGripEnabled(true);
    setModal(true);

    absolute_check();

    QGridLayout	*layout = new QGridLayout(this);
    layout->addWidget(setup_position(),0,0);
    layout->addLayout(setup_buttons(),0,1);
    layout->setSpacing(6);
    layout->setMargin(8);
    //QObject::connect(lista, SIGNAL(itemSelectionChanged()), this, SLOT(selection_changed()));
}

void	win_move_to::absolute_check()
{
    if (GV->selected_comp.count() == 1)
        if (GV->selected_obj.count() == 0)
        {
            show_absolute = TRUE;
            absolute_x = GV->selected_comp.first()->anchor_point.x();
            absolute_y = GV->selected_comp.first()->anchor_point.y();
            if (y_axis_is_up)
                absolute_y = -absolute_y;
        }

    if (GV->selected_comp.count() == 0)
        if (GV->selected_obj.count() == 1)
            if ((GV->selected_obj.first()->tipo == O_PAD) || (GV->selected_obj.first()->tipo == O_VIA))
            {
                show_absolute = TRUE;
                absolute_x = GV->selected_obj.first()->node_start->point.x();
                absolute_y = GV->selected_obj.first()->node_start->point.y();
                if (y_axis_is_up)
                    absolute_y = -absolute_y;
            }
}

QGroupBox		*win_move_to::setup_position()
{
    QGroupBox *groupBox = new QGroupBox(tr("Position"));

    if (show_absolute)
    {
        is_relative = new QCheckBox(tr("Relative Position"));
        connect(is_relative,SIGNAL(stateChanged(int)),this,SLOT(change_is_relative(int)));
    }
    p_x = new QLabel(this);
    p_x->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
    pos_x = new layout_length("", 0, -2000000, 2000000, FALSE);
    p_y = new QLabel(this);
    p_y->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
    pos_y = new layout_length("", 0, -2000000, 2000000, FALSE);
    QGridLayout *gbox = new QGridLayout();
    int row = 0;
    if (show_absolute)
    {
        gbox->addWidget(is_relative,row++,0, TRUE, FALSE);
        is_relative->setChecked(TRUE);
    }
    change_is_relative(TRUE);
    gbox->addWidget(p_x,row,0);
    gbox->addLayout(pos_x,row++,1);
    gbox->addWidget(p_y,row,0);
    gbox->addLayout(pos_y,row,1);
    groupBox->setLayout(gbox);

    return groupBox;
}

QVBoxLayout	*win_move_to::setup_buttons()
{
    QVBoxLayout	*L_comandi = new QVBoxLayout();
    L_comandi->setSpacing(6);
    L_comandi->setMargin(0);
    QPushButton	*L_comandi_drag_move = new QPushButton(this);
    L_comandi_drag_move->setText(tr("Drag"));
    L_comandi->addWidget(L_comandi_drag_move);
    QPushButton	*L_comandi_duplicate = new QPushButton(this);
    L_comandi_duplicate->setText(tr("Duplicate"));
    L_comandi->addWidget(L_comandi_duplicate);
    QSpacerItem	*L_comandi_molla = new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);
    L_comandi->addItem(L_comandi_molla);
    QPushButton	*L_comandi_cancel = new QPushButton(this);
    L_comandi_cancel->setText(tr("Cancel"));
    L_comandi->addWidget(L_comandi_cancel);

    QObject::connect(L_comandi_drag_move, SIGNAL(clicked()), this, SLOT(drag_move()));
    QObject::connect(L_comandi_duplicate, SIGNAL(clicked()), this, SLOT(duplicate()));
    QObject::connect(L_comandi_cancel, SIGNAL(clicked()), this, SLOT(reject()));

    L_comandi_drag_move->setAutoDefault(FALSE);
    L_comandi_duplicate->setAutoDefault(FALSE);

    return L_comandi;
}

void	win_move_to::change_is_relative(int new_value)
{
    if (new_value)
    {
        //Movimento relativo
        p_x->setText("      " + QString(&QChar(916),1) + "X");
        p_y->setText("      " + QString(&QChar(916),1) + "Y");
        pos_x->setLength(0);
        pos_y->setLength(0);
    }
    else
    {
        //Movimento assoluto
        p_x->setText("      X");
        p_y->setText("      Y");
        pos_x->setLength(absolute_x);
        pos_y->setLength(absolute_y);
    }
}

void	win_move_to::drag_move()
{
    bool    relative;
    if (show_absolute)
        relative = is_relative->isChecked();
    else
        relative = TRUE;

    int x_move = pos_x->length;
    int y_move = pos_y->length;
    if (y_axis_is_up)
    {
        absolute_y = -absolute_y;
        y_move = -y_move;
    }
    if (!relative)
    {
        x_move = x_move - absolute_x;
        y_move = y_move - absolute_y;
    }

    GV->setUpdatesEnabled(FALSE);
    drag_or_move_delta.setX(x_move);
    drag_or_move_delta.setY(y_move);
    if (!drag_or_move_delta.isNull())
    {
        if (mode_drag_active)
        {
            GV->drag_or_move_setup('D');
            GV->drag_or_move_execute('D');
        }
        else
        {
            GV->drag_or_move_setup('M');
            GV->drag_or_move_execute('M');
        }
        GV->drag_or_move_setup('R');
    }
    GV->setUpdatesEnabled(TRUE);

    //Aggiorno il valore di absolute_x e absolute_y
    absolute_check();
}

void	win_move_to::duplicate()
{
    bool    relative;
    if (show_absolute)
        relative = is_relative->isChecked();
    else
        relative = TRUE;

    int x_move = pos_x->length;
    int y_move = pos_y->length;
    if (y_axis_is_up)
    {
        absolute_y = -absolute_y;
        y_move = -y_move;
    }
    if (!relative)
    {
        x_move = x_move - absolute_x;
        y_move = y_move - absolute_y;
    }

    GV->setUpdatesEnabled(FALSE);
    GV->clone_from_selection();
    GV->clone_ident_pin_pad_recalculate();
    GV->select_clear();
    GV->drag_or_move_setup('R');
    GV->clone_move(QPoint(x_move, y_move), TRUE);
    GV->ur.beginMacro(tr("Duplicate"));
    GV->clone_to_selection();
    GV->clone_from_selection();
    GV->clone_ident_pin_pad_recalculate();
    if (GV->is_sch)
        w_main->connect_sch_selection();
    GV->ur.endMacro();
    clipboard_obj.clear();
    clipboard_comp.clear();
    clipboard_nodes.clear();
    GV->tool_stop();
    GV->setUpdatesEnabled(TRUE);

    //Aggiorno il valore di absolute_x e absolute_y
    absolute_check();

    if ((x_move == 0) && (y_move == 0))
        QMessageBox::warning(0,tr("Warning"),tr("Duplication at same position."));
}
